import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import _objectWithoutProperties from "@babel/runtime/helpers/esm/objectWithoutProperties";
const _excluded = ["class", "style"];
import { createVNode as _createVNode, resolveDirective as _resolveDirective } from "vue";
import { ref, computed, watchEffect, defineComponent } from 'vue';
import PropTypes from '../_util/vue-types';
import warning from '../_util/warning';
import classNames from '../_util/classNames';
import SlickCarousel from '../vc-slick';
import { withInstall } from '../_util/type';
import useConfigInject from '../_util/hooks/useConfigInject';
// Carousel
export const carouselProps = () => ({
  effect: String,
  dots: {
    type: Boolean,
    default: true
  },
  vertical: {
    type: Boolean,
    default: undefined
  },
  autoplay: {
    type: Boolean,
    default: undefined
  },
  easing: String,
  beforeChange: Function,
  afterChange: Function,
  // style: PropTypes.React.CSSProperties,
  prefixCls: String,
  accessibility: {
    type: Boolean,
    default: undefined
  },
  nextArrow: PropTypes.any,
  prevArrow: PropTypes.any,
  pauseOnHover: {
    type: Boolean,
    default: undefined
  },
  // className: String,
  adaptiveHeight: {
    type: Boolean,
    default: undefined
  },
  arrows: {
    type: Boolean,
    default: false
  },
  autoplaySpeed: Number,
  centerMode: {
    type: Boolean,
    default: undefined
  },
  centerPadding: String,
  cssEase: String,
  dotsClass: String,
  draggable: {
    type: Boolean,
    default: false
  },
  fade: {
    type: Boolean,
    default: undefined
  },
  focusOnSelect: {
    type: Boolean,
    default: undefined
  },
  infinite: {
    type: Boolean,
    default: undefined
  },
  initialSlide: Number,
  lazyLoad: String,
  rtl: {
    type: Boolean,
    default: undefined
  },
  slide: String,
  slidesToShow: Number,
  slidesToScroll: Number,
  speed: Number,
  swipe: {
    type: Boolean,
    default: undefined
  },
  swipeToSlide: {
    type: Boolean,
    default: undefined
  },
  swipeEvent: Function,
  touchMove: {
    type: Boolean,
    default: undefined
  },
  touchThreshold: Number,
  variableWidth: {
    type: Boolean,
    default: undefined
  },
  useCSS: {
    type: Boolean,
    default: undefined
  },
  slickGoTo: Number,
  responsive: Array,
  dotPosition: {
    type: String,
    default: undefined
  },
  verticalSwiping: {
    type: Boolean,
    default: false
  }
});
const Carousel = defineComponent({
  compatConfig: {
    MODE: 3
  },
  name: 'ACarousel',
  inheritAttrs: false,
  props: carouselProps(),
  setup(props, _ref) {
    let {
      slots,
      attrs,
      expose
    } = _ref;
    const slickRef = ref();
    const goTo = function (slide) {
      var _slickRef$value;
      let dontAnimate = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
      (_slickRef$value = slickRef.value) === null || _slickRef$value === void 0 ? void 0 : _slickRef$value.slickGoTo(slide, dontAnimate);
    };
    expose({
      goTo,
      autoplay: palyType => {
        var _slickRef$value2, _slickRef$value2$inne;
        (_slickRef$value2 = slickRef.value) === null || _slickRef$value2 === void 0 ? void 0 : (_slickRef$value2$inne = _slickRef$value2.innerSlider) === null || _slickRef$value2$inne === void 0 ? void 0 : _slickRef$value2$inne.handleAutoPlay(palyType);
      },
      prev: () => {
        var _slickRef$value3;
        (_slickRef$value3 = slickRef.value) === null || _slickRef$value3 === void 0 ? void 0 : _slickRef$value3.slickPrev();
      },
      next: () => {
        var _slickRef$value4;
        (_slickRef$value4 = slickRef.value) === null || _slickRef$value4 === void 0 ? void 0 : _slickRef$value4.slickNext();
      },
      innerSlider: computed(() => {
        var _slickRef$value5;
        return (_slickRef$value5 = slickRef.value) === null || _slickRef$value5 === void 0 ? void 0 : _slickRef$value5.innerSlider;
      })
    });
    watchEffect(() => {
      warning(props.vertical === undefined, 'Carousel', '`vertical` is deprecated, please use `dotPosition` instead.');
    });
    const {
      prefixCls,
      direction
    } = useConfigInject('carousel', props);
    const dotPosition = computed(() => {
      if (props.dotPosition) return props.dotPosition;
      if (props.vertical !== undefined) return props.vertical ? 'right' : 'bottom';
      return 'bottom';
    });
    const vertical = computed(() => dotPosition.value === 'left' || dotPosition.value === 'right');
    const dsClass = computed(() => {
      const dotsClass = 'slick-dots';
      return classNames({
        [dotsClass]: true,
        [`${dotsClass}-${dotPosition.value}`]: true,
        [`${props.dotsClass}`]: !!props.dotsClass
      });
    });
    return () => {
      const {
        dots,
        arrows,
        draggable,
        effect
      } = props;
      const {
          class: cls,
          style
        } = attrs,
        restAttrs = _objectWithoutProperties(attrs, _excluded);
      const fade = effect === 'fade' ? true : props.fade;
      const className = classNames(prefixCls.value, {
        [`${prefixCls.value}-rtl`]: direction.value === 'rtl',
        [`${prefixCls.value}-vertical`]: vertical.value,
        [`${cls}`]: !!cls
      });
      return _createVNode("div", {
        "class": className,
        "style": style
      }, [_createVNode(SlickCarousel, _objectSpread(_objectSpread(_objectSpread({
        "ref": slickRef
      }, props), restAttrs), {}, {
        "dots": !!dots,
        "dotsClass": dsClass.value,
        "arrows": arrows,
        "draggable": draggable,
        "fade": fade,
        "vertical": vertical.value
      }), slots)]);
    };
  }
});
export default withInstall(Carousel);