import { createVNode as _createVNode } from "vue";
import { defineComponent } from 'vue';
import PropTypes from '../_util/vue-types';
import { flattenChildren } from '../_util/props-util';
import { withInstall } from '../_util/type';
import useConfigInject from '../_util/hooks/useConfigInject';
export const commentProps = () => ({
  actions: Array,
  /** The element to display as the comment author. */
  author: PropTypes.any,
  /** The element to display as the comment avatar - generally an antd Avatar */
  avatar: PropTypes.any,
  /** The main content of the comment */
  content: PropTypes.any,
  /** Comment prefix defaults to '.ant-comment' */
  prefixCls: String,
  /** A datetime element containing the time to be displayed */
  datetime: PropTypes.any
});
const Comment = defineComponent({
  compatConfig: {
    MODE: 3
  },
  name: 'AComment',
  props: commentProps(),
  slots: Object,
  setup(props, _ref) {
    let {
      slots
    } = _ref;
    const {
      prefixCls,
      direction
    } = useConfigInject('comment', props);
    const renderNested = (prefixCls, children) => {
      return _createVNode("div", {
        "class": `${prefixCls}-nested`
      }, [children]);
    };
    const getAction = actions => {
      if (!actions || !actions.length) {
        return null;
      }
      const actionList = actions.map((action, index) => _createVNode("li", {
        "key": `action-${index}`
      }, [action]));
      return actionList;
    };
    return () => {
      var _slots$actions, _slots$author, _slots$avatar, _slots$content, _slots$datetime, _slots$default;
      const pre = prefixCls.value;
      const actions = props.actions ?? ((_slots$actions = slots.actions) === null || _slots$actions === void 0 ? void 0 : _slots$actions.call(slots));
      const author = props.author ?? ((_slots$author = slots.author) === null || _slots$author === void 0 ? void 0 : _slots$author.call(slots));
      const avatar = props.avatar ?? ((_slots$avatar = slots.avatar) === null || _slots$avatar === void 0 ? void 0 : _slots$avatar.call(slots));
      const content = props.content ?? ((_slots$content = slots.content) === null || _slots$content === void 0 ? void 0 : _slots$content.call(slots));
      const datetime = props.datetime ?? ((_slots$datetime = slots.datetime) === null || _slots$datetime === void 0 ? void 0 : _slots$datetime.call(slots));
      const avatarDom = _createVNode("div", {
        "class": `${pre}-avatar`
      }, [typeof avatar === 'string' ? _createVNode("img", {
        "src": avatar,
        "alt": "comment-avatar"
      }, null) : avatar]);
      const actionDom = actions ? _createVNode("ul", {
        "class": `${pre}-actions`
      }, [getAction(Array.isArray(actions) ? actions : [actions])]) : null;
      const authorContent = _createVNode("div", {
        "class": `${pre}-content-author`
      }, [author && _createVNode("span", {
        "class": `${pre}-content-author-name`
      }, [author]), datetime && _createVNode("span", {
        "class": `${pre}-content-author-time`
      }, [datetime])]);
      const contentDom = _createVNode("div", {
        "class": `${pre}-content`
      }, [authorContent, _createVNode("div", {
        "class": `${pre}-content-detail`
      }, [content]), actionDom]);
      const comment = _createVNode("div", {
        "class": `${pre}-inner`
      }, [avatarDom, contentDom]);
      const children = flattenChildren((_slots$default = slots.default) === null || _slots$default === void 0 ? void 0 : _slots$default.call(slots));
      return _createVNode("div", {
        "class": [pre, {
          [`${pre}-rtl`]: direction.value === 'rtl'
        }]
      }, [comment, children && children.length ? renderNested(pre, children) : null]);
    };
  }
});
export default withInstall(Comment);