import _objectWithoutProperties from "@babel/runtime/helpers/esm/objectWithoutProperties";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
const _excluded = ["placement", "overlayClassName"];
import { resolveDirective as _resolveDirective, createVNode as _createVNode } from "vue";
import { computed, onMounted, ref, toRef, defineComponent } from 'vue';
import Tooltip from '../tooltip';
import abstractTooltipProps from '../tooltip/abstractTooltipProps';
import PropTypes from '../_util/vue-types';
import { initDefaultProps } from '../_util/props-util';
import { convertLegacyProps } from '../button/buttonTypes';
import ExclamationCircleFilled from "@ant-design/icons-vue/es/icons/ExclamationCircleFilled";
import Button from '../button';
import { useLocaleReceiver } from '../locale-provider/LocaleReceiver';
import defaultLocale from '../locale-provider/default';
import { withInstall } from '../_util/type';
import useMergedState from '../_util/hooks/useMergedState';
import devWarning from '../vc-util/devWarning';
import KeyCode from '../_util/KeyCode';
import useConfigInject from '../_util/hooks/useConfigInject';
import classNames from '../_util/classNames';
import { getTransitionName } from '../_util/transition';
import { cloneVNodes } from '../_util/vnode';
import omit from '../_util/omit';
import { tooltipDefaultProps } from '../tooltip/Tooltip';
import ActionButton from '../_util/ActionButton';
export const popconfirmProps = () => _objectSpread(_objectSpread({}, abstractTooltipProps()), {}, {
  prefixCls: String,
  content: PropTypes.any,
  title: PropTypes.any,
  okType: {
    type: String,
    default: 'primary'
  },
  disabled: {
    type: Boolean,
    default: false
  },
  okText: PropTypes.any,
  cancelText: PropTypes.any,
  icon: PropTypes.any,
  okButtonProps: {
    type: Object,
    default: undefined
  },
  cancelButtonProps: {
    type: Object,
    default: undefined
  },
  showCancel: {
    type: Boolean,
    default: true
  },
  onConfirm: Function,
  onCancel: Function
});
const Popconfirm = defineComponent({
  compatConfig: {
    MODE: 3
  },
  name: 'APopconfirm',
  props: initDefaultProps(popconfirmProps(), _objectSpread(_objectSpread({}, tooltipDefaultProps()), {}, {
    trigger: 'click',
    transitionName: 'zoom-big',
    placement: 'top',
    mouseEnterDelay: 0.1,
    mouseLeaveDelay: 0.1,
    arrowPointAtCenter: false,
    autoAdjustOverflow: true,
    okType: 'primary',
    disabled: false
  })),
  slots: Object,
  emits: ['update:visible', 'visibleChange'],
  setup(props, _ref) {
    let {
      slots,
      emit,
      expose
    } = _ref;
    onMounted(() => {
      devWarning(props.defaultVisible === undefined, 'Popconfirm', `'defaultVisible' is deprecated, please use 'v-model:visible'`);
    });
    const tooltipRef = ref();
    expose({
      getPopupDomNode: () => {
        var _tooltipRef$value, _tooltipRef$value$get;
        return (_tooltipRef$value = tooltipRef.value) === null || _tooltipRef$value === void 0 ? void 0 : (_tooltipRef$value$get = _tooltipRef$value.getPopupDomNode) === null || _tooltipRef$value$get === void 0 ? void 0 : _tooltipRef$value$get.call(_tooltipRef$value);
      }
    });
    const [visible, setVisible] = useMergedState(false, {
      value: toRef(props, 'visible'),
      defaultValue: props.defaultVisible
    });
    const settingVisible = (value, e) => {
      if (props.visible === undefined) {
        setVisible(value);
      }
      emit('update:visible', value);
      emit('visibleChange', value, e);
    };
    const close = e => {
      settingVisible(false, e);
    };
    const onConfirm = e => {
      var _props$onConfirm;
      return (_props$onConfirm = props.onConfirm) === null || _props$onConfirm === void 0 ? void 0 : _props$onConfirm.call(props, e);
    };
    const onCancel = e => {
      var _props$onCancel;
      settingVisible(false, e);
      (_props$onCancel = props.onCancel) === null || _props$onCancel === void 0 ? void 0 : _props$onCancel.call(props, e);
    };
    const onKeyDown = e => {
      if (e.keyCode === KeyCode.ESC && visible) {
        settingVisible(false, e);
      }
    };
    const onVisibleChange = value => {
      const {
        disabled
      } = props;
      if (disabled) {
        return;
      }
      settingVisible(value);
    };
    const {
      prefixCls: prefixClsConfirm,
      getPrefixCls
    } = useConfigInject('popconfirm', props);
    const rootPrefixCls = computed(() => getPrefixCls());
    const popoverPrefixCls = computed(() => getPrefixCls('popover'));
    const btnPrefixCls = computed(() => getPrefixCls('btn'));
    const [popconfirmLocale] = useLocaleReceiver('Popconfirm', defaultLocale.Popconfirm);
    const renderOverlay = () => {
      var _slots$title, _slots$cancel, _slots$okText, _slots$icon;
      const {
        okButtonProps,
        cancelButtonProps,
        title = (_slots$title = slots.title) === null || _slots$title === void 0 ? void 0 : _slots$title.call(slots),
        cancelText = (_slots$cancel = slots.cancel) === null || _slots$cancel === void 0 ? void 0 : _slots$cancel.call(slots),
        okText = (_slots$okText = slots.okText) === null || _slots$okText === void 0 ? void 0 : _slots$okText.call(slots),
        okType,
        icon = (_slots$icon = slots.icon) === null || _slots$icon === void 0 ? void 0 : _slots$icon.call(slots),
        showCancel = true
      } = props;
      const {
        cancelButton,
        okButton
      } = slots;
      const cancelProps = _objectSpread({
        onClick: onCancel,
        size: 'small'
      }, cancelButtonProps);
      const okProps = _objectSpread(_objectSpread({
        onClick: onConfirm
      }, convertLegacyProps(okType)), {}, {
        size: 'small'
      }, okButtonProps);
      return _createVNode("div", {
        "class": `${popoverPrefixCls.value}-inner-content`
      }, [_createVNode("div", {
        "class": `${popoverPrefixCls.value}-message`
      }, [icon || _createVNode(ExclamationCircleFilled, null, null), _createVNode("div", {
        "class": `${popoverPrefixCls.value}-message-title`
      }, [title])]), _createVNode("div", {
        "class": `${popoverPrefixCls.value}-buttons`
      }, [showCancel ? cancelButton ? cancelButton(cancelProps) : _createVNode(Button, cancelProps, {
        default: () => [cancelText || popconfirmLocale.value.cancelText]
      }) : null, okButton ? okButton(okProps) : _createVNode(ActionButton, {
        "buttonProps": _objectSpread(_objectSpread({
          size: 'small'
        }, convertLegacyProps(okType)), okButtonProps),
        "actionFn": onConfirm,
        "close": close,
        "prefixCls": btnPrefixCls.value,
        "quitOnNullishReturnValue": true,
        "emitEvent": true
      }, {
        default: () => [okText || popconfirmLocale.value.okText]
      })])]);
    };
    return () => {
      var _slots$default;
      const {
          placement,
          overlayClassName
        } = props,
        restProps = _objectWithoutProperties(props, _excluded);
      const otherProps = omit(restProps, ['title', 'content', 'cancelText', 'okText', 'onUpdate:visible', 'onConfirm', 'onCancel']);
      const overlayClassNames = classNames(prefixClsConfirm.value, overlayClassName);
      return _createVNode(Tooltip, _objectSpread(_objectSpread({}, otherProps), {}, {
        "prefixCls": popoverPrefixCls.value,
        "placement": placement,
        "onVisibleChange": onVisibleChange,
        "visible": visible.value,
        "overlayClassName": overlayClassNames,
        "transitionName": getTransitionName(rootPrefixCls.value, 'zoom-big', props.transitionName),
        "ref": tooltipRef
      }), {
        default: () => [cloneVNodes(((_slots$default = slots.default) === null || _slots$default === void 0 ? void 0 : _slots$default.call(slots)) || [], {
          onKeydown: e => {
            onKeyDown(e);
          }
        }, false)],
        title: renderOverlay
      });
    };
  }
});
export default withInstall(Popconfirm);