import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import { createVNode as _createVNode } from "vue";
import { defineComponent } from 'vue';
import PropTypes from '../_util/vue-types';
import initDefaultProps from '../_util/props-util/initDefaultProps';
import StatisticNumber from './Number';
import Skeleton from '../skeleton/Skeleton';
import useConfigInject from '../_util/hooks/useConfigInject';
export const statisticProps = () => ({
  prefixCls: String,
  decimalSeparator: String,
  groupSeparator: String,
  format: String,
  value: {
    type: [String, Number, Object]
  },
  valueStyle: {
    type: Object,
    default: undefined
  },
  valueRender: PropTypes.any,
  formatter: PropTypes.any,
  precision: Number,
  prefix: PropTypes.any,
  suffix: PropTypes.any,
  title: PropTypes.any,
  loading: {
    type: Boolean,
    default: undefined
  }
});
export default defineComponent({
  compatConfig: {
    MODE: 3
  },
  name: 'AStatistic',
  props: initDefaultProps(statisticProps(), {
    decimalSeparator: '.',
    groupSeparator: ',',
    loading: false
  }),
  slots: Object,
  setup(props, _ref) {
    let {
      slots
    } = _ref;
    const {
      prefixCls,
      direction
    } = useConfigInject('statistic', props);
    return () => {
      var _slots$title, _slots$prefix, _slots$suffix;
      const {
        value = 0,
        valueStyle,
        valueRender
      } = props;
      const pre = prefixCls.value;
      const title = props.title ?? ((_slots$title = slots.title) === null || _slots$title === void 0 ? void 0 : _slots$title.call(slots));
      const prefix = props.prefix ?? ((_slots$prefix = slots.prefix) === null || _slots$prefix === void 0 ? void 0 : _slots$prefix.call(slots));
      const suffix = props.suffix ?? ((_slots$suffix = slots.suffix) === null || _slots$suffix === void 0 ? void 0 : _slots$suffix.call(slots));
      const formatter = props.formatter ?? slots.formatter;
      // data-for-update just for update component
      // https://github.com/vueComponent/ant-design-vue/pull/3170
      let valueNode = _createVNode(StatisticNumber, _objectSpread({
        "data-for-update": Date.now()
      }, _objectSpread(_objectSpread({}, props), {}, {
        prefixCls: pre,
        value,
        formatter
      })), null);
      if (valueRender) {
        valueNode = valueRender(valueNode);
      }
      return _createVNode("div", {
        "class": [pre, {
          [`${pre}-rtl`]: direction.value === 'rtl'
        }]
      }, [title && _createVNode("div", {
        "class": `${pre}-title`
      }, [title]), _createVNode(Skeleton, {
        "paragraph": false,
        "loading": props.loading
      }, {
        default: () => [_createVNode("div", {
          "style": valueStyle,
          "class": `${pre}-content`
        }, [prefix && _createVNode("span", {
          "class": `${pre}-content-prefix`
        }, [prefix]), valueNode, suffix && _createVNode("span", {
          "class": `${pre}-content-suffix`
        }, [suffix])])]
      })]);
    };
  }
});