import { createVNode as _createVNode } from "vue";
import { defineComponent } from 'vue';
import classNames from '../_util/classNames';
import PropTypes from '../_util/vue-types';
import initDefaultProps from '../_util/props-util/initDefaultProps';
import { tuple } from '../_util/type';
import useConfigInject from '../_util/hooks/useConfigInject';
export const timelineItemProps = () => ({
  prefixCls: String,
  color: String,
  dot: PropTypes.any,
  pending: {
    type: Boolean,
    default: undefined
  },
  position: PropTypes.oneOf(tuple('left', 'right', '')).def(''),
  label: PropTypes.any
});
export default defineComponent({
  compatConfig: {
    MODE: 3
  },
  name: 'ATimelineItem',
  props: initDefaultProps(timelineItemProps(), {
    color: 'blue',
    pending: false
  }),
  slots: Object,
  setup(props, _ref) {
    let {
      slots
    } = _ref;
    const {
      prefixCls
    } = useConfigInject('timeline', props);
    return () => {
      var _slots$label, _slots$dot, _slots$default;
      const {
        color = '',
        pending,
        label = (_slots$label = slots.label) === null || _slots$label === void 0 ? void 0 : _slots$label.call(slots),
        dot = (_slots$dot = slots.dot) === null || _slots$dot === void 0 ? void 0 : _slots$dot.call(slots)
      } = props;
      const itemClassName = classNames({
        [`${prefixCls.value}-item`]: true,
        [`${prefixCls.value}-item-pending`]: pending
      });
      const dotClassName = classNames({
        [`${prefixCls.value}-item-head`]: true,
        [`${prefixCls.value}-item-head-custom`]: dot,
        [`${prefixCls.value}-item-head-${color}`]: true
      });
      const customColor = /blue|red|green|gray/.test(color || '') ? undefined : color;
      return _createVNode("li", {
        "class": itemClassName
      }, [label && _createVNode("div", {
        "class": `${prefixCls.value}-item-label`
      }, [label]), _createVNode("div", {
        "class": `${prefixCls.value}-item-tail`
      }, null), _createVNode("div", {
        "class": dotClassName,
        "style": {
          borderColor: customColor,
          color: customColor
        }
      }, [dot]), _createVNode("div", {
        "class": `${prefixCls.value}-item-content`
      }, [(_slots$default = slots.default) === null || _slots$default === void 0 ? void 0 : _slots$default.call(slots)])]);
    };
  }
});