import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import { createVNode as _createVNode } from "vue";
import PropTypes, { withUndefined } from '../_util/vue-types';
import { defineComponent } from 'vue';
function isString(str) {
  return typeof str === 'string';
}
function noop() {}
export const VcStepProps = () => ({
  prefixCls: String,
  wrapperStyle: {
    type: Object,
    default: undefined
  },
  itemWidth: String,
  active: {
    type: Boolean,
    default: undefined
  },
  disabled: {
    type: Boolean,
    default: undefined
  },
  status: String,
  iconPrefix: String,
  icon: PropTypes.any,
  adjustMarginRight: String,
  stepNumber: Number,
  stepIndex: Number,
  description: PropTypes.any,
  title: PropTypes.any,
  subTitle: PropTypes.any,
  progressDot: withUndefined(PropTypes.oneOfType([PropTypes.looseBool, PropTypes.func])),
  tailContent: PropTypes.any,
  icons: PropTypes.shape({
    finish: PropTypes.any,
    error: PropTypes.any
  }).loose,
  onClick: Function,
  onStepClick: Function,
  stepIcon: Function
});
export default defineComponent({
  compatConfig: {
    MODE: 3
  },
  name: 'Step',
  props: VcStepProps(),
  emits: ['click', 'stepClick'],
  setup(props, _ref) {
    let {
      slots,
      emit
    } = _ref;
    const onItemClick = e => {
      emit('click', e);
      emit('stepClick', props.stepIndex);
    };
    const renderIconNode = _ref2 => {
      let {
        icon,
        title,
        description
      } = _ref2;
      const {
        prefixCls,
        stepNumber,
        status,
        iconPrefix,
        icons,
        progressDot = slots.progressDot,
        stepIcon = slots.stepIcon
      } = props;
      let iconNode;
      const iconClassName = {
        [`${prefixCls}-icon`]: true,
        [`${iconPrefix}icon`]: true,
        [`${iconPrefix}icon-${icon}`]: icon && isString(icon),
        [`${iconPrefix}icon-check`]: !icon && status === 'finish' && icons && !icons.finish,
        [`${iconPrefix}icon-close`]: !icon && status === 'error' && icons && !icons.error
      };
      const iconDot = _createVNode("span", {
        "class": `${prefixCls}-icon-dot`
      }, null);
      // `progressDot` enjoy the highest priority
      if (progressDot) {
        if (typeof progressDot === 'function') {
          iconNode = _createVNode("span", {
            "class": `${prefixCls}-icon`
          }, [progressDot({
            iconDot,
            index: stepNumber - 1,
            status,
            title,
            description,
            prefixCls
          })]);
        } else {
          iconNode = _createVNode("span", {
            "class": `${prefixCls}-icon`
          }, [iconDot]);
        }
      } else if (icon && !isString(icon)) {
        iconNode = _createVNode("span", {
          "class": `${prefixCls}-icon`
        }, [icon]);
      } else if (icons && icons.finish && status === 'finish') {
        iconNode = _createVNode("span", {
          "class": `${prefixCls}-icon`
        }, [icons.finish]);
      } else if (icons && icons.error && status === 'error') {
        iconNode = _createVNode("span", {
          "class": `${prefixCls}-icon`
        }, [icons.error]);
      } else if (icon || status === 'finish' || status === 'error') {
        iconNode = _createVNode("span", {
          "class": iconClassName
        }, null);
      } else {
        iconNode = _createVNode("span", {
          "class": `${prefixCls}-icon`
        }, [stepNumber]);
      }
      if (stepIcon) {
        iconNode = stepIcon({
          index: stepNumber - 1,
          status,
          title,
          description,
          node: iconNode
        });
      }
      return iconNode;
    };
    return () => {
      var _slots$title, _slots$description, _slots$subTitle, _slots$icon;
      const {
        prefixCls,
        itemWidth,
        active,
        status = 'wait',
        tailContent,
        adjustMarginRight,
        disabled,
        title = (_slots$title = slots.title) === null || _slots$title === void 0 ? void 0 : _slots$title.call(slots),
        description = (_slots$description = slots.description) === null || _slots$description === void 0 ? void 0 : _slots$description.call(slots),
        subTitle = (_slots$subTitle = slots.subTitle) === null || _slots$subTitle === void 0 ? void 0 : _slots$subTitle.call(slots),
        icon = (_slots$icon = slots.icon) === null || _slots$icon === void 0 ? void 0 : _slots$icon.call(slots),
        onClick,
        onStepClick
      } = props;
      const classString = {
        [`${prefixCls}-item`]: true,
        [`${prefixCls}-item-${status}`]: true,
        [`${prefixCls}-item-custom`]: icon,
        [`${prefixCls}-item-active`]: active,
        [`${prefixCls}-item-disabled`]: disabled === true
      };
      const stepProps = {
        class: classString
      };
      const stepItemStyle = {};
      if (itemWidth) {
        stepItemStyle.width = itemWidth;
      }
      if (adjustMarginRight) {
        stepItemStyle.marginRight = adjustMarginRight;
      }
      const accessibilityProps = {
        onClick: onClick || noop
      };
      if (onStepClick && !disabled) {
        accessibilityProps.role = 'button';
        accessibilityProps.tabindex = 0;
        accessibilityProps.onClick = onItemClick;
      }
      return _createVNode("div", _objectSpread(_objectSpread({}, stepProps), {}, {
        "style": stepItemStyle
      }), [_createVNode("div", _objectSpread(_objectSpread({}, accessibilityProps), {}, {
        "class": `${prefixCls}-item-container`
      }), [_createVNode("div", {
        "class": `${prefixCls}-item-tail`
      }, [tailContent]), _createVNode("div", {
        "class": `${prefixCls}-item-icon`
      }, [renderIconNode({
        icon,
        title,
        description
      })]), _createVNode("div", {
        "class": `${prefixCls}-item-content`
      }, [_createVNode("div", {
        "class": `${prefixCls}-item-title`
      }, [title, subTitle && _createVNode("div", {
        "title": subTitle,
        "class": `${prefixCls}-item-subtitle`
      }, [subTitle])]), description && _createVNode("div", {
        "class": `${prefixCls}-item-description`
      }, [description])])])]);
    };
  }
});