"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _vue = require("vue");
var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread2"));
var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));
var _vueTypes = _interopRequireDefault(require("../_util/vue-types"));
var _configProvider = require("../config-provider");
var _BaseMixin = _interopRequireDefault(require("../_util/BaseMixin"));
var _pickrEs = _interopRequireDefault(require("@simonwep/pickr/dist/pickr.es5.min"));
var _icon = _interopRequireDefault(require("../icon"));
var _LocaleReceiver = _interopRequireDefault(require("../locale-provider/LocaleReceiver"));
var _en_US = _interopRequireDefault(require("./locale/en_US"));
var _debounce = _interopRequireDefault(require("lodash/debounce"));
var _propsUtil = require("../_util/props-util");
/* eslint-disable */

let colors = '#194d33';
var _default = exports.default = {
  name: 'AColorPicker',
  mixins: [_BaseMixin.default],
  inject: {
    configProvider: {
      default: () => _configProvider.defaultConfigProvider
    }
  },
  model: {
    prop: 'value',
    event: 'change.value' //为了支持v-model直接返回颜色字符串 所以用了自定义的事件,与pickr自带change事件进行区分
  },
  props: {
    prefixCls: String,
    defaultValue: String,
    //默认值
    config: _vueTypes.default.object,
    //pickr配置
    value: String,
    //颜色值
    locale: _vueTypes.default.object,
    //双语包
    colorRounded: Number,
    //颜色数值保留几位小数
    size: _vueTypes.default.oneOf(['default', 'small', 'large']).def('default'),
    //尺寸
    getPopupContainer: Function,
    //指定渲染容器
    disabled: {
      type: Boolean,
      default: false
    },
    //是否禁用
    format: String,
    //颜色格式设置
    alpha: {
      type: Boolean,
      default: false
    },
    //是否开启透明通道
    hue: {
      type: Boolean,
      default: true
    } //是否开启色彩预选
  },
  data() {
    return {
      colors,
      myOpen: false,
      pickr: null,
      i18n: _en_US.default
    };
  },
  watch: {
    'configProvider.locale.ColorPicker': {
      handler(val) {
        if (this.locale) return;
        this.i18n = val;
        this.reInitialize();
      }
    },
    locale(val) {
      this.i18n = val.ColorPicker || val.lang;
      this.reInitialize();
    },
    value(val) {
      this.setColor(val);
    },
    disabled(val) {
      this.pickr[val ? 'disable' : 'enable']();
    },
    config: {
      handler() {
        this.reInitialize();
      },
      deep: true
    },
    format(val) {
      const type = val.toLocaleUpperCase();
      let res = this.pickr.setColorRepresentation(type);
      if (res) {
        this.pickr.applyColor();
      } else {
        throw new TypeError('format was invalid');
      }
    }
  },
  mounted() {
    if (this.locale) {
      this.i18n = this.locale.ColorPicker || this.locale.lang;
    }
    this.createPickr();
    this.eventsBinding();
  },
  unmounted() {
    this.pickr.destroyAndRemove();
  },
  methods: {
    reInitialize() {
      this.pickr.destroyAndRemove();
      const dom = document.createElement('div');
      dom.id = 'color-picker' + this._uid;
      const box = (0, _propsUtil.findDOMNode)(this).querySelector('#color-picker-box' + this._uid);
      box.appendChild(dom);
      this.createPickr();
      this.eventsBinding();
    },
    setColor: (0, _debounce.default)(function (val) {
      this.pickr.setColor(val);
    }, 1000),
    eventsBinding() {
      const pickrEvents = ['init', 'hide', 'show', 'save', 'clear', 'change', 'changestop', 'cancel', 'swatchselect'];
      Object.keys(this.$listeners).forEach(event => {
        pickrEvents.includes(event) && this.pickr.on(event, this.$listeners[event]);
      });
    },
    createPickr() {
      const {
        getPopupContainer
      } = (0, _propsUtil.getOptionProps)(this);
      const {
        getPopupContainer: getContextPopupContainer
      } = this.configProvider;
      const container = getPopupContainer || getContextPopupContainer;
      this.pickr = _pickrEs.default.create((0, _extends2.default)({
        el: '#color-picker' + this._uid,
        container: container && container((0, _propsUtil.findDOMNode)(this)) || document.body,
        theme: 'monolith',
        // or 'monolith', or 'nano'
        default: this.value || this.defaultValue || null,
        // 有默认颜色pickr才可以获取到_representation
        components: {
          // Main components
          preview: true,
          opacity: this.alpha,
          hue: this.hue,
          // Input / output Options
          interaction: {
            hex: true,
            rgba: true,
            input: true,
            clear: true,
            save: true
          }
        }
      }, this.config, {
        i18n: this.i18n
      })).on('save', (color, instance) => {
        if (color) {
          let _representation = instance._representation || 'HEXA';
          color = color['to' + _representation]().toString(this.colorRounded || 0);
        }
        this.$emit('change.value', color || '');
      }).on('hide', () => {
        this.setState({
          myOpen: false
        });
      });
    },
    handleOpenChange() {
      const open = !this.myOpen;
      this.setState({
        myOpen: open
      });
      this.pickr[open ? 'show' : 'hide']();
      this.$emit('openChange', open);
    },
    getDefaultLocale() {
      const result = (0, _objectSpread2.default)((0, _objectSpread2.default)({}, _en_US.default), this.$props.locale);
      result.lang = (0, _objectSpread2.default)((0, _objectSpread2.default)({}, result.lang), (this.$props.locale || {}).lang);
      return result;
    },
    renderColorPicker() {
      const {
        prefixCls: customizePrefixCls
      } = this.$props;
      const {
        getPrefixCls
      } = this.configProvider;
      const prefixCls = getPrefixCls('color-picker', customizePrefixCls);
      const {
        disabled
      } = (0, _propsUtil.getOptionProps)(this);
      const classString = {
        [prefixCls]: true,
        [`${prefixCls}-open`]: this.myOpen,
        [`${prefixCls}-lg`]: this.size === 'large',
        [`${prefixCls}-sm`]: this.size === 'small',
        [`${prefixCls}-disabled`]: this.disabled
      };
      return (0, _vue.createVNode)("div", {
        "class": classString,
        "tabindex": disabled ? -1 : 0,
        "onClick": this.handleOpenChange
      }, [(0, _vue.createVNode)("div", {
        "class": `${prefixCls}-selection`
      }, [(0, _vue.createVNode)("div", {
        "id": 'color-picker-box' + this._uid
      }, [(0, _vue.createVNode)("div", {
        "id": 'color-picker' + this._uid
      }, null)]), (0, _vue.createVNode)(_icon.default, {
        "type": "down",
        "class": `${prefixCls}-icon`
      }, null)])]);
    }
  },
  render() {
    return (0, _vue.createVNode)(_LocaleReceiver.default, {
      "componentName": "ColorPicker",
      "defaultLocale": this.getDefaultLocale,
      "scopedSlots": {
        default: this.renderColorPicker
      }
    }, null);
  }
};