"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.commentProps = void 0;
var _vue = require("vue");
var _vueTypes = _interopRequireDefault(require("../_util/vue-types"));
var _propsUtil = require("../_util/props-util");
var _type = require("../_util/type");
var _useConfigInject = _interopRequireDefault(require("../_util/hooks/useConfigInject"));
const commentProps = () => ({
  actions: Array,
  /** The element to display as the comment author. */
  author: _vueTypes.default.any,
  /** The element to display as the comment avatar - generally an antd Avatar */
  avatar: _vueTypes.default.any,
  /** The main content of the comment */
  content: _vueTypes.default.any,
  /** Comment prefix defaults to '.ant-comment' */
  prefixCls: String,
  /** A datetime element containing the time to be displayed */
  datetime: _vueTypes.default.any
});
exports.commentProps = commentProps;
const Comment = (0, _vue.defineComponent)({
  compatConfig: {
    MODE: 3
  },
  name: 'AComment',
  props: commentProps(),
  slots: Object,
  setup(props, _ref) {
    let {
      slots
    } = _ref;
    const {
      prefixCls,
      direction
    } = (0, _useConfigInject.default)('comment', props);
    const renderNested = (prefixCls, children) => {
      return (0, _vue.createVNode)("div", {
        "class": `${prefixCls}-nested`
      }, [children]);
    };
    const getAction = actions => {
      if (!actions || !actions.length) {
        return null;
      }
      const actionList = actions.map((action, index) => (0, _vue.createVNode)("li", {
        "key": `action-${index}`
      }, [action]));
      return actionList;
    };
    return () => {
      var _slots$actions, _slots$author, _slots$avatar, _slots$content, _slots$datetime, _slots$default;
      const pre = prefixCls.value;
      const actions = props.actions ?? ((_slots$actions = slots.actions) === null || _slots$actions === void 0 ? void 0 : _slots$actions.call(slots));
      const author = props.author ?? ((_slots$author = slots.author) === null || _slots$author === void 0 ? void 0 : _slots$author.call(slots));
      const avatar = props.avatar ?? ((_slots$avatar = slots.avatar) === null || _slots$avatar === void 0 ? void 0 : _slots$avatar.call(slots));
      const content = props.content ?? ((_slots$content = slots.content) === null || _slots$content === void 0 ? void 0 : _slots$content.call(slots));
      const datetime = props.datetime ?? ((_slots$datetime = slots.datetime) === null || _slots$datetime === void 0 ? void 0 : _slots$datetime.call(slots));
      const avatarDom = (0, _vue.createVNode)("div", {
        "class": `${pre}-avatar`
      }, [typeof avatar === 'string' ? (0, _vue.createVNode)("img", {
        "src": avatar,
        "alt": "comment-avatar"
      }, null) : avatar]);
      const actionDom = actions ? (0, _vue.createVNode)("ul", {
        "class": `${pre}-actions`
      }, [getAction(Array.isArray(actions) ? actions : [actions])]) : null;
      const authorContent = (0, _vue.createVNode)("div", {
        "class": `${pre}-content-author`
      }, [author && (0, _vue.createVNode)("span", {
        "class": `${pre}-content-author-name`
      }, [author]), datetime && (0, _vue.createVNode)("span", {
        "class": `${pre}-content-author-time`
      }, [datetime])]);
      const contentDom = (0, _vue.createVNode)("div", {
        "class": `${pre}-content`
      }, [authorContent, (0, _vue.createVNode)("div", {
        "class": `${pre}-content-detail`
      }, [content]), actionDom]);
      const comment = (0, _vue.createVNode)("div", {
        "class": `${pre}-inner`
      }, [avatarDom, contentDom]);
      const children = (0, _propsUtil.flattenChildren)((_slots$default = slots.default) === null || _slots$default === void 0 ? void 0 : _slots$default.call(slots));
      return (0, _vue.createVNode)("div", {
        "class": [pre, {
          [`${pre}-rtl`]: direction.value === 'rtl'
        }]
      }, [comment, children && children.length ? renderNested(pre, children) : null]);
    };
  }
});
var _default = exports.default = (0, _type.withInstall)(Comment);