import type { PropType, ExtractPropTypes, HTMLAttributes, ComponentPublicInstance } from 'vue';
import FormItem from './FormItem';
import type { Options } from 'scroll-into-view-if-needed';
import type { ColProps } from '../grid/Col';
import type { InternalNamePath, NamePath, ValidateOptions, Callbacks, ValidateMessages, Rule, FormLabelAlign } from './interface';
import type { SizeType } from '../config-provider';
import useForm from './useForm';
export type RequiredMark = boolean | 'optional';
export type FormLayout = 'horizontal' | 'inline' | 'vertical';
export declare const formProps: () => {
    layout: import("vue-types").VueTypeDef<FormLayout>;
    labelCol: {
        type: PropType<ColProps & HTMLAttributes>;
    };
    wrapperCol: {
        type: PropType<ColProps & HTMLAttributes>;
    };
    colon: {
        type: BooleanConstructor;
        default: any;
    };
    labelAlign: import("vue-types").VueTypeDef<FormLabelAlign>;
    labelWrap: {
        type: BooleanConstructor;
        default: any;
    };
    prefixCls: StringConstructor;
    requiredMark: {
        type: PropType<RequiredMark | "">;
        default: any;
    };
    /** @deprecated Will warning in future branch. Pls use `requiredMark` instead. */
    hideRequiredMark: {
        type: BooleanConstructor;
        default: any;
    };
    model: import("vue-types").VueTypeValidableDef<{
        [key: string]: any;
    }> & {
        default: () => {
            [key: string]: any;
        };
    };
    rules: {
        type: PropType<{
            [k: string]: Rule[] | Rule;
        }>;
    };
    validateMessages: {
        type: PropType<ValidateMessages>;
        default: ValidateMessages;
    };
    validateOnRuleChange: {
        type: BooleanConstructor;
        default: any;
    };
    scrollToFirstError: {
        type: PropType<boolean | Options>;
    };
    onSubmit: PropType<(e: Event) => void>;
    name: StringConstructor;
    validateTrigger: {
        type: PropType<string | string[]>;
    };
    size: {
        type: PropType<SizeType>;
    };
    onValuesChange: {
        type: PropType<Callbacks["onValuesChange"]>;
    };
    onFieldsChange: {
        type: PropType<Callbacks["onFieldsChange"]>;
    };
    onFinish: {
        type: PropType<Callbacks["onFinish"]>;
    };
    onFinishFailed: {
        type: PropType<Callbacks["onFinishFailed"]>;
    };
    onValidate: {
        type: PropType<Callbacks["onValidate"]>;
    };
};
export type FormProps = Partial<ExtractPropTypes<ReturnType<typeof formProps>>>;
export type FormExpose = {
    resetFields: (name?: NamePath) => void;
    clearValidate: (name?: NamePath) => void;
    validateFields: (nameList?: NamePath[] | string, options?: ValidateOptions) => Promise<{
        [key: string]: any;
    }>;
    getFieldsValue: (nameList?: InternalNamePath[] | true) => {
        [key: string]: any;
    };
    validate: (nameList?: NamePath[] | string, options?: ValidateOptions) => Promise<{
        [key: string]: any;
    }>;
    scrollToField: (name: NamePath, options?: {}) => void;
};
export type FormInstance = ComponentPublicInstance<FormProps, FormExpose>;
declare const Form: import("vue").DefineComponent<{
    layout: import("vue-types").VueTypeDef<FormLayout>;
    labelCol: {
        type: PropType<ColProps & HTMLAttributes>;
    };
    wrapperCol: {
        type: PropType<ColProps & HTMLAttributes>;
    };
    colon: {
        type: BooleanConstructor;
        default: any;
    };
    labelAlign: import("vue-types").VueTypeDef<FormLabelAlign>;
    labelWrap: {
        type: BooleanConstructor;
        default: any;
    };
    prefixCls: StringConstructor;
    requiredMark: {
        type: PropType<RequiredMark | "">;
        default: any;
    };
    /** @deprecated Will warning in future branch. Pls use `requiredMark` instead. */
    hideRequiredMark: {
        type: BooleanConstructor;
        default: any;
    };
    model: import("vue-types").VueTypeValidableDef<{
        [key: string]: any;
    }> & {
        default: () => {
            [key: string]: any;
        };
    };
    rules: {
        type: PropType<{
            [k: string]: Rule[] | Rule;
        }>;
    };
    validateMessages: {
        type: PropType<ValidateMessages>;
        default: ValidateMessages;
    };
    validateOnRuleChange: {
        type: BooleanConstructor;
        default: any;
    };
    scrollToFirstError: {
        type: PropType<boolean | Options>;
    };
    onSubmit: PropType<(e: Event) => void>;
    name: StringConstructor;
    validateTrigger: {
        type: PropType<string | string[]>;
    };
    size: {
        type: PropType<SizeType>;
    };
    onValuesChange: {
        type: PropType<Callbacks["onValuesChange"]>;
    };
    onFieldsChange: {
        type: PropType<Callbacks["onFieldsChange"]>;
    };
    onFinish: {
        type: PropType<Callbacks["onFinish"]>;
    };
    onFinishFailed: {
        type: PropType<Callbacks["onFinishFailed"]>;
    };
    onValidate: {
        type: PropType<Callbacks["onValidate"]>;
    };
}, () => import("vue/jsx-runtime").JSX.Element, unknown, {}, {}, import("vue").ComponentOptionsMixin, import("vue").ComponentOptionsMixin, {}, string, import("vue").PublicProps, Readonly<ExtractPropTypes<{
    layout: import("vue-types").VueTypeDef<FormLayout>;
    labelCol: {
        type: PropType<ColProps & HTMLAttributes>;
    };
    wrapperCol: {
        type: PropType<ColProps & HTMLAttributes>;
    };
    colon: {
        type: BooleanConstructor;
        default: any;
    };
    labelAlign: import("vue-types").VueTypeDef<FormLabelAlign>;
    labelWrap: {
        type: BooleanConstructor;
        default: any;
    };
    prefixCls: StringConstructor;
    requiredMark: {
        type: PropType<RequiredMark | "">;
        default: any;
    };
    /** @deprecated Will warning in future branch. Pls use `requiredMark` instead. */
    hideRequiredMark: {
        type: BooleanConstructor;
        default: any;
    };
    model: import("vue-types").VueTypeValidableDef<{
        [key: string]: any;
    }> & {
        default: () => {
            [key: string]: any;
        };
    };
    rules: {
        type: PropType<{
            [k: string]: Rule[] | Rule;
        }>;
    };
    validateMessages: {
        type: PropType<ValidateMessages>;
        default: ValidateMessages;
    };
    validateOnRuleChange: {
        type: BooleanConstructor;
        default: any;
    };
    scrollToFirstError: {
        type: PropType<boolean | Options>;
    };
    onSubmit: PropType<(e: Event) => void>;
    name: StringConstructor;
    validateTrigger: {
        type: PropType<string | string[]>;
    };
    size: {
        type: PropType<SizeType>;
    };
    onValuesChange: {
        type: PropType<Callbacks["onValuesChange"]>;
    };
    onFieldsChange: {
        type: PropType<Callbacks["onFieldsChange"]>;
    };
    onFinish: {
        type: PropType<Callbacks["onFinish"]>;
    };
    onFinishFailed: {
        type: PropType<Callbacks["onFinishFailed"]>;
    };
    onValidate: {
        type: PropType<Callbacks["onValidate"]>;
    };
}>>, {
    colon: boolean;
    requiredMark: "" | RequiredMark;
    labelWrap: boolean;
    validateMessages: ValidateMessages;
    hideRequiredMark: boolean;
    model: {
        [key: string]: any;
    };
    validateOnRuleChange: boolean;
}, {}>;
declare const _default: typeof Form & {
    readonly Item: typeof FormItem;
    readonly useForm: typeof useForm;
};
export default _default;
