"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
Object.defineProperty(exports, "ListItem", {
  enumerable: true,
  get: function () {
    return _Item.default;
  }
});
Object.defineProperty(exports, "ListItemMeta", {
  enumerable: true,
  get: function () {
    return _ItemMeta.default;
  }
});
exports.listProps = exports.default = void 0;
var _vue = require("vue");
var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread2"));
var _vueTypes = _interopRequireDefault(require("../_util/vue-types"));
var _spin = _interopRequireDefault(require("../spin"));
var _pagination = _interopRequireDefault(require("../pagination"));
var _grid = require("../grid");
var _Item = _interopRequireDefault(require("./Item"));
var _propsUtil = require("../_util/props-util");
var _initDefaultProps = _interopRequireDefault(require("../_util/props-util/initDefaultProps"));
var _ItemMeta = _interopRequireDefault(require("./ItemMeta"));
var _useConfigInject = _interopRequireDefault(require("../_util/hooks/useConfigInject"));
var _useBreakpoint = _interopRequireDefault(require("../_util/hooks/useBreakpoint"));
var _responsiveObserve = require("../_util/responsiveObserve");
var _eagerComputed = _interopRequireDefault(require("../_util/eagerComputed"));
var _contextKey = require("./contextKey");
const listProps = () => ({
  bordered: {
    type: Boolean,
    default: undefined
  },
  dataSource: _vueTypes.default.array,
  extra: _vueTypes.default.any,
  grid: {
    type: Object,
    default: undefined
  },
  itemLayout: String,
  loading: {
    type: [Boolean, Object],
    default: undefined
  },
  loadMore: _vueTypes.default.any,
  pagination: {
    type: [Boolean, Object],
    default: undefined
  },
  prefixCls: String,
  rowKey: [String, Number, Function],
  renderItem: Function,
  size: String,
  split: {
    type: Boolean,
    default: undefined
  },
  header: _vueTypes.default.any,
  footer: _vueTypes.default.any,
  locale: {
    type: Object
  }
});
exports.listProps = listProps;
const List = (0, _vue.defineComponent)({
  compatConfig: {
    MODE: 3
  },
  name: 'AList',
  Item: _Item.default,
  props: (0, _initDefaultProps.default)(listProps(), {
    dataSource: [],
    bordered: false,
    split: true,
    loading: false,
    pagination: false
  }),
  slots: Object,
  setup(props, _ref) {
    let {
      slots
    } = _ref;
    (0, _vue.provide)(_contextKey.ListContextKey, {
      grid: (0, _vue.toRef)(props, 'grid'),
      itemLayout: (0, _vue.toRef)(props, 'itemLayout')
    });
    const defaultPaginationProps = {
      current: 1,
      total: 0
    };
    const {
      prefixCls,
      direction,
      renderEmpty
    } = (0, _useConfigInject.default)('list', props);
    const paginationObj = (0, _vue.computed)(() => props.pagination && typeof props.pagination === 'object' ? props.pagination : {});
    const paginationCurrent = (0, _vue.ref)(paginationObj.value.defaultCurrent ?? 1);
    const paginationSize = (0, _vue.ref)(paginationObj.value.defaultPageSize ?? 10);
    (0, _vue.watch)(paginationObj, () => {
      if ('current' in paginationObj.value) {
        paginationCurrent.value = paginationObj.value.current;
      }
      if ('pageSize' in paginationObj.value) {
        paginationSize.value = paginationObj.value.pageSize;
      }
    });
    const listItemsKeys = [];
    const triggerPaginationEvent = eventName => (page, pageSize) => {
      paginationCurrent.value = page;
      paginationSize.value = pageSize;
      if (paginationObj.value[eventName]) {
        paginationObj.value[eventName](page, pageSize);
      }
    };
    const onPaginationChange = triggerPaginationEvent('onChange');
    const onPaginationShowSizeChange = triggerPaginationEvent('onShowSizeChange');
    const renderEmptyFunc = renderEmptyHandler => {
      var _props$locale;
      return (0, _vue.createVNode)("div", {
        "class": `${prefixCls.value}-empty-text`
      }, [((_props$locale = props.locale) === null || _props$locale === void 0 ? void 0 : _props$locale.emptyText) || renderEmptyHandler('List')]);
    };
    const loadingProp = (0, _vue.computed)(() => {
      if (typeof props.loading === 'boolean') {
        return {
          spinning: props.loading
        };
      } else {
        return props.loading;
      }
    });
    const isLoading = (0, _vue.computed)(() => loadingProp.value && loadingProp.value.spinning);
    const sizeCls = (0, _vue.computed)(() => {
      let size = '';
      switch (props.size) {
        case 'large':
          size = 'lg';
          break;
        case 'small':
          size = 'sm';
          break;
        default:
          break;
      }
      return size;
    });
    const classObj = (0, _vue.computed)(() => ({
      [`${prefixCls.value}`]: true,
      [`${prefixCls.value}-vertical`]: props.itemLayout === 'vertical',
      [`${prefixCls.value}-${sizeCls.value}`]: sizeCls.value,
      [`${prefixCls.value}-split`]: props.split,
      [`${prefixCls.value}-bordered`]: props.bordered,
      [`${prefixCls.value}-loading`]: isLoading.value,
      [`${prefixCls.value}-grid`]: !!props.grid,
      [`${prefixCls.value}-rtl`]: direction.value === 'rtl'
    }));
    const paginationProps = (0, _vue.computed)(() => {
      const pp = (0, _objectSpread2.default)((0, _objectSpread2.default)({}, defaultPaginationProps), {}, {
        total: props.dataSource.length,
        current: paginationCurrent.value,
        pageSize: paginationSize.value
      }, props.pagination || {});
      const largestPage = Math.ceil(pp.total / pp.pageSize);
      if (pp.current > largestPage) {
        pp.current = largestPage;
      }
      return pp;
    });
    const splitDataSource = (0, _vue.computed)(() => {
      let dd = [...props.dataSource];
      if (props.pagination) {
        if (props.dataSource.length > (paginationProps.value.current - 1) * paginationProps.value.pageSize) {
          dd = [...props.dataSource].splice((paginationProps.value.current - 1) * paginationProps.value.pageSize, paginationProps.value.pageSize);
        }
      }
      return dd;
    });
    const screens = (0, _useBreakpoint.default)();
    const currentBreakpoint = (0, _eagerComputed.default)(() => {
      for (let i = 0; i < _responsiveObserve.responsiveArray.length; i += 1) {
        const breakpoint = _responsiveObserve.responsiveArray[i];
        if (screens.value[breakpoint]) {
          return breakpoint;
        }
      }
      return undefined;
    });
    const colStyle = (0, _vue.computed)(() => {
      if (!props.grid) {
        return undefined;
      }
      const columnCount = currentBreakpoint.value && props.grid[currentBreakpoint.value] ? props.grid[currentBreakpoint.value] : props.grid.column;
      if (columnCount) {
        return {
          width: `${100 / columnCount}%`,
          maxWidth: `${100 / columnCount}%`
        };
      }
      return undefined;
    });
    const renderInnerItem = (item, index) => {
      const renderItem = props.renderItem ?? slots.renderItem;
      if (!renderItem) return null;
      let key;
      const rowKeyType = typeof props.rowKey;
      if (rowKeyType === 'function') {
        key = props.rowKey(item);
      } else if (rowKeyType === 'string' || rowKeyType === 'number') {
        key = item[props.rowKey];
      } else {
        key = item.key;
      }
      if (!key) {
        key = `list-item-${index}`;
      }
      listItemsKeys[index] = key;
      return renderItem({
        item,
        index
      });
    };
    return () => {
      var _slots$loadMore, _slots$footer, _slots$header, _slots$default;
      const loadMore = props.loadMore ?? ((_slots$loadMore = slots.loadMore) === null || _slots$loadMore === void 0 ? void 0 : _slots$loadMore.call(slots));
      const footer = props.footer ?? ((_slots$footer = slots.footer) === null || _slots$footer === void 0 ? void 0 : _slots$footer.call(slots));
      const header = props.header ?? ((_slots$header = slots.header) === null || _slots$header === void 0 ? void 0 : _slots$header.call(slots));
      const children = (0, _propsUtil.flattenChildren)((_slots$default = slots.default) === null || _slots$default === void 0 ? void 0 : _slots$default.call(slots));
      const isSomethingAfterLastItem = !!(loadMore || props.pagination || footer);
      const classString = (0, _objectSpread2.default)((0, _objectSpread2.default)({}, classObj.value), {}, {
        [`${prefixCls.value}-something-after-last-item`]: isSomethingAfterLastItem
      });
      const paginationContent = props.pagination ? (0, _vue.createVNode)("div", {
        "class": `${prefixCls.value}-pagination`
      }, [(0, _vue.createVNode)(_pagination.default, (0, _objectSpread2.default)((0, _objectSpread2.default)({}, paginationProps.value), {}, {
        "onChange": onPaginationChange,
        "onShowSizeChange": onPaginationShowSizeChange
      }), null)]) : null;
      let childrenContent = isLoading.value && (0, _vue.createVNode)("div", {
        "style": {
          minHeight: '53px'
        }
      }, null);
      if (splitDataSource.value.length > 0) {
        listItemsKeys.length = 0;
        const items = splitDataSource.value.map((item, index) => renderInnerItem(item, index));
        const childrenList = items.map((child, index) => (0, _vue.createVNode)("div", {
          "key": listItemsKeys[index],
          "style": colStyle.value
        }, [child]));
        childrenContent = props.grid ? (0, _vue.createVNode)(_grid.Row, {
          "gutter": props.grid.gutter
        }, {
          default: () => [childrenList]
        }) : (0, _vue.createVNode)("ul", {
          "class": `${prefixCls.value}-items`
        }, [items]);
      } else if (!children.length && !isLoading.value) {
        childrenContent = renderEmptyFunc(renderEmpty.value);
      }
      const paginationPosition = paginationProps.value.position || 'bottom';
      return (0, _vue.createVNode)("div", {
        "class": classString
      }, [(paginationPosition === 'top' || paginationPosition === 'both') && paginationContent, header && (0, _vue.createVNode)("div", {
        "class": `${prefixCls.value}-header`
      }, [header]), (0, _vue.createVNode)(_spin.default, loadingProp.value, {
        default: () => [childrenContent, children]
      }), footer && (0, _vue.createVNode)("div", {
        "class": `${prefixCls.value}-footer`
      }, [footer]), loadMore || (paginationPosition === 'bottom' || paginationPosition === 'both') && paginationContent]);
    };
  }
});
/* istanbul ignore next */
List.install = function (app) {
  app.component(List.name, List);
  app.component(List.Item.name, List.Item);
  app.component(List.Item.Meta.name, List.Item.Meta);
  return app;
};
var _default = exports.default = List;