"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
exports.setDefaultIndicator = setDefaultIndicator;
exports.spinProps = void 0;
var _vue = require("vue");
var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread2"));
var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));
var _debounce = _interopRequireDefault(require("lodash/debounce"));
var _vueTypes = _interopRequireDefault(require("../_util/vue-types"));
var _propsUtil = require("../_util/props-util");
var _initDefaultProps = _interopRequireDefault(require("../_util/props-util/initDefaultProps"));
var _configProvider = require("../config-provider");
const _excluded = ["class", "style"];
const spinProps = () => ({
  prefixCls: String,
  spinning: {
    type: Boolean,
    default: undefined
  },
  size: String,
  wrapperClassName: String,
  tip: _vueTypes.default.any,
  delay: Number,
  indicator: _vueTypes.default.any
});
// Render indicator
exports.spinProps = spinProps;
let defaultIndicator = null;
function shouldDelay(spinning, delay) {
  return !!spinning && !!delay && !isNaN(Number(delay));
}
function setDefaultIndicator(Content) {
  const Indicator = Content.indicator;
  defaultIndicator = typeof Indicator === 'function' ? Indicator : () => (0, _vue.createVNode)(Indicator, null, null);
}
var _default = exports.default = (0, _vue.defineComponent)({
  compatConfig: {
    MODE: 3
  },
  name: 'ASpin',
  inheritAttrs: false,
  props: (0, _initDefaultProps.default)(spinProps(), {
    size: 'default',
    spinning: true,
    wrapperClassName: ''
  }),
  setup() {
    return {
      originalUpdateSpinning: null,
      configProvider: (0, _vue.inject)('configProvider', _configProvider.defaultConfigProvider)
    };
  },
  data() {
    const {
      spinning,
      delay
    } = this;
    const shouldBeDelayed = shouldDelay(spinning, delay);
    return {
      sSpinning: spinning && !shouldBeDelayed
    };
  },
  created() {
    this.originalUpdateSpinning = this.updateSpinning;
    this.debouncifyUpdateSpinning(this.$props);
  },
  mounted() {
    this.updateSpinning();
  },
  updated() {
    (0, _vue.nextTick)(() => {
      this.debouncifyUpdateSpinning();
      this.updateSpinning();
    });
  },
  beforeUnmount() {
    this.cancelExistingSpin();
  },
  methods: {
    debouncifyUpdateSpinning(props) {
      const {
        delay
      } = props || this.$props;
      if (delay) {
        this.cancelExistingSpin();
        this.updateSpinning = (0, _debounce.default)(this.originalUpdateSpinning, delay);
      }
    },
    updateSpinning() {
      const {
        spinning,
        sSpinning
      } = this;
      if (sSpinning !== spinning) {
        this.sSpinning = spinning;
      }
    },
    cancelExistingSpin() {
      const {
        updateSpinning
      } = this;
      if (updateSpinning && updateSpinning.cancel) {
        updateSpinning.cancel();
      }
    },
    renderIndicator(prefixCls) {
      const dotClassName = `${prefixCls}-dot`;
      let indicator = (0, _propsUtil.getComponent)(this, 'indicator');
      // should not be render default indicator when indicator value is null
      if (indicator === null) {
        return null;
      }
      if (Array.isArray(indicator)) {
        indicator = indicator.length === 1 ? indicator[0] : indicator;
      }
      if ((0, _vue.isVNode)(indicator)) {
        return (0, _vue.cloneVNode)(indicator, {
          class: dotClassName
        });
      }
      if (defaultIndicator && (0, _vue.isVNode)(defaultIndicator())) {
        return (0, _vue.cloneVNode)(defaultIndicator(), {
          class: dotClassName
        });
      }
      return (0, _vue.createVNode)("span", {
        "class": `${dotClassName} ${prefixCls}-dot-spin`
      }, [(0, _vue.createVNode)("i", {
        "class": `${prefixCls}-dot-item`
      }, null), (0, _vue.createVNode)("i", {
        "class": `${prefixCls}-dot-item`
      }, null), (0, _vue.createVNode)("i", {
        "class": `${prefixCls}-dot-item`
      }, null), (0, _vue.createVNode)("i", {
        "class": `${prefixCls}-dot-item`
      }, null)]);
    }
  },
  render() {
    var _this$$slots$tip, _this$$slots;
    const {
      size,
      prefixCls: customizePrefixCls,
      tip = (_this$$slots$tip = (_this$$slots = this.$slots).tip) === null || _this$$slots$tip === void 0 ? void 0 : _this$$slots$tip.call(_this$$slots),
      wrapperClassName
    } = this.$props;
    const _this$$attrs = this.$attrs,
      {
        class: cls,
        style
      } = _this$$attrs,
      divProps = (0, _objectWithoutProperties2.default)(_this$$attrs, _excluded);
    const {
      getPrefixCls,
      direction
    } = this.configProvider;
    const prefixCls = getPrefixCls('spin', customizePrefixCls);
    const {
      sSpinning
    } = this;
    const spinClassName = {
      [prefixCls]: true,
      [`${prefixCls}-sm`]: size === 'small',
      [`${prefixCls}-lg`]: size === 'large',
      [`${prefixCls}-spinning`]: sSpinning,
      [`${prefixCls}-show-text`]: !!tip,
      [`${prefixCls}-rtl`]: direction === 'rtl',
      [cls]: !!cls
    };
    const spinElement = (0, _vue.createVNode)("div", (0, _objectSpread2.default)((0, _objectSpread2.default)({}, divProps), {}, {
      "style": style,
      "class": spinClassName
    }), [this.renderIndicator(prefixCls), tip ? (0, _vue.createVNode)("div", {
      "class": `${prefixCls}-text`
    }, [tip]) : null]);
    const children = (0, _propsUtil.getSlot)(this);
    if (children && children.length) {
      const containerClassName = {
        [`${prefixCls}-container`]: true,
        [`${prefixCls}-blur`]: sSpinning
      };
      return (0, _vue.createVNode)("div", {
        "class": [`${prefixCls}-nested-loading`, wrapperClassName]
      }, [sSpinning && (0, _vue.createVNode)("div", {
        "key": "loading"
      }, [spinElement]), (0, _vue.createVNode)("div", {
        "class": containerClassName,
        "key": "container"
      }, [children])]);
    }
    return spinElement;
  }
});